import {
  ui,
  forwardRef,
  HTMLUIProps,
  ThemeProps,
  useMultiComponentStyle,
  omitThemeProps,
  CSSUIObject,
} from '@yamada-ui/core'
import { InfoIcon, WarningIcon, CheckIcon } from '@yamada-ui/icon'
import { Loading, LoadingProps } from '@yamada-ui/loading'
import { createContext, cx } from '@yamada-ui/utils'
import { FC } from 'react'

const statuses = {
  info: { icon: InfoIcon, colorScheme: 'primary' },
  success: { icon: CheckIcon, colorScheme: 'secondary' },
  warning: { icon: WarningIcon, colorScheme: 'warning' },
  error: { icon: WarningIcon, colorScheme: 'danger' },
  loading: { icon: Loading, colorScheme: 'primary' },
} as const

type Status = keyof typeof statuses

type AlertContext = {
  status: Status
  styles: Record<string, CSSUIObject>
}

const [AlertProvider, useAlert] = createContext<AlertContext>({
  name: `AlertStylesContext`,
  errorMessage: `useAlert returned is 'undefined'. Seems you forgot to wrap the components in "<Alert />" `,
})

export const getStatusColorScheme = (status: Status) => statuses[status].colorScheme

export const getStatusIcon = (status: Status) => statuses[status].icon

type AlertOptions = {
  /**
   * The status of the alert.
   *
   * @default 'info'
   */
  status?: Status
}

export type AlertProps = HTMLUIProps<'div'> & ThemeProps<'Alert'> & AlertOptions

export const Alert = forwardRef<AlertProps, 'div'>(
  ({ status = 'info', colorScheme, ...props }, ref) => {
    colorScheme = colorScheme ?? getStatusColorScheme(status)

    const [styles, mergedProps] = useMultiComponentStyle('Alert', { ...props, colorScheme })
    const { className, children, ...rest } = omitThemeProps(mergedProps)

    const css: CSSUIObject = {
      w: '100%',
      display: 'flex',
      alignItems: 'center',
      position: 'relative',
      overflow: 'hidden',
      ...styles.container,
    }

    return (
      <AlertProvider value={{ status, styles }}>
        <ui.div ref={ref} className={cx('ui-alert', className)} role='alert' __css={css} {...rest}>
          {children}
        </ui.div>
      </AlertProvider>
    )
  },
)

export type AlertIconProps = HTMLUIProps<'span'> & { variant?: LoadingProps['variant'] }

export const AlertIcon: FC<AlertIconProps> = ({
  className,
  children,
  variant = 'oval',
  ...rest
}) => {
  const { status, styles } = useAlert()

  const Icon = getStatusIcon(status)
  const css: CSSUIObject = {
    ...(status === 'loading' ? styles.loading : styles.icon),
  }

  return (
    <ui.span display='inherit' className={cx('ui-alert-icon', className)} __css={css} {...rest}>
      {children || (
        <Icon
          {...(status === 'loading' ? { variant, color: 'currentcolor' } : { boxSize: '100%' })}
        />
      )}
    </ui.span>
  )
}

export type AlertTitleProps = HTMLUIProps<'p'>

export const AlertTitle = forwardRef<AlertTitleProps, 'p'>(({ className, ...rest }, ref) => {
  const { styles } = useAlert()

  const css: CSSUIObject = {
    display: 'block',
    ...styles.title,
  }

  return <ui.p ref={ref} className={cx('ui-alert-title', className)} __css={css} {...rest} />
})

export type AlertDescriptionProps = HTMLUIProps<'span'>

export const AlertDescription = forwardRef<AlertDescriptionProps, 'span'>(
  ({ className, ...rest }, ref) => {
    const { styles } = useAlert()

    const css: CSSUIObject = {
      ...styles.description,
    }

    return <ui.span ref={ref} className={cx('ui-alert-desc', className)} __css={css} {...rest} />
  },
)
